<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @package helpful-pro
 */

/**
 * This class should be used to work with the administrative side of WordPress.
 */
class daexthefup_Admin {

	/**
	 * The single instance of the class.
	 *
	 * @var null
	 */
	protected static $instance = null;

	/**
	 * An instance of the $shared class.
	 *
	 * @var daexthefup_Shared|null
	 */
	private $shared = null;

	private $screen_id_statistics  = null;
	private $screen_id_maintenance = null;
	private $screen_id_help        = null;
	private $screen_id_options     = null;

	public $menu_options = null;

	private function __construct() {

		// assign an instance of the plugin info.
		$this->shared = daexthefup_Shared::get_instance();

		// Load admin stylesheets and JavaScript.
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_styles' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ) );

		// Add the admin menu.
		add_action( 'admin_menu', array( $this, 'me_add_admin_menu' ) );

		// Require and instantiate the class used to register the menu options.
		require_once $this->shared->get( 'dir' ) . 'admin/inc/class-daexthefup-menu-options.php';
		$this->menu_options = new Daexthefup_Menu_Options( $this->shared );

		// Require and instantiate the class used to handle the CSV exports.
		require_once $this->shared->get( 'dir' ) . 'admin/inc/class-daexthefup-export-controller.php';
		new Daexthefup_Export_Controller( $this->shared );

		// Load the options API registrations and callbacks.
		add_action( 'admin_init', array( $this, 'op_register_options' ) );

		// Add custom columns to all the post types with a public UI.
		add_action( 'admin_init', array( $this, 'add_custom_columns' ) );
	}

	/**
	 * return an instance of this class
	 *
	 * @return self|null
	 */
	public static function get_instance() {

		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Enqueue admin stylesheets.
	 *
	 * @return void
	 */
	public function enqueue_admin_styles() {

		$screen = get_current_screen();

		wp_enqueue_style(
			$this->shared->get( 'slug' ) . '-menu-posts',
			$this->shared->get( 'url' ) . 'admin/assets/css/menu-posts.css',
			array(),
			$this->shared->get( 'ver' )
		);

		// menu statistics.
		if ( $screen->id === $this->screen_id_statistics ) {

			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-framework-menu',
				$this->shared->get( 'url' ) . 'admin/assets/css/framework/menu.css',
				array(),
				$this->shared->get( 'ver' )
			);
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-menu-dashboard',
				$this->shared->get( 'url' ) . 'admin/assets/css/menu-statistics.css',
				array(),
				$this->shared->get( 'ver' )
			);
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-jquery-ui-tooltip',
				$this->shared->get( 'url' ) . 'admin/assets/css/jquery-ui-tooltip.css',
				array(),
				$this->shared->get( 'ver' )
			);

			// jQuery UI Dialog.
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-jquery-ui-dialog',
				$this->shared->get( 'url' ) . 'admin/assets/css/jquery-ui-dialog.css',
				array(),
				$this->shared->get( 'ver' )
			);
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-jquery-ui-dialog-custom',
				$this->shared->get( 'url' ) . 'admin/assets/css/jquery-ui-dialog-custom.css',
				array(),
				$this->shared->get( 'ver' )
			);

			// Select2.
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-select2',
				$this->shared->get( 'url' ) . 'admin/assets/inc/select2/css/select2.min.css',
				array(),
				$this->shared->get( 'ver' )
			);
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-select2-custom',
				$this->shared->get( 'url' ) . 'admin/assets/css/select2-custom.css',
				array(),
				$this->shared->get( 'ver' )
			);

		}

		// menu options.
		if ( $screen->id === $this->screen_id_maintenance ) {

			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-framework-menu',
				$this->shared->get( 'url' ) . 'admin/assets/css/framework/menu.css',
				array(),
				$this->shared->get( 'ver' )
			);
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-jquery-ui-tooltip',
				$this->shared->get( 'url' ) . 'admin/assets/css/jquery-ui-tooltip.css',
				array(),
				$this->shared->get( 'ver' )
			);

			// jQuery UI Dialog.
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-jquery-ui-dialog',
				$this->shared->get( 'url' ) . 'admin/assets/css/jquery-ui-dialog.css',
				array(),
				$this->shared->get( 'ver' )
			);
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-jquery-ui-dialog-custom',
				$this->shared->get( 'url' ) . 'admin/assets/css/jquery-ui-dialog-custom.css',
				array(),
				$this->shared->get( 'ver' )
			);

			// Select2.
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-select2',
				$this->shared->get( 'url' ) . 'admin/assets/inc/select2/css/select2.min.css',
				array(),
				$this->shared->get( 'ver' )
			);
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-select2-custom',
				$this->shared->get( 'url' ) . 'admin/assets/css/select2-custom.css',
				array(),
				$this->shared->get( 'ver' )
			);

		}

		// Menu Help.
		if ( $screen->id === $this->screen_id_help ) {

			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-menu-help',
				$this->shared->get( 'url' ) . 'admin/assets/css/menu-help.css',
				array(),
				$this->shared->get( 'ver' )
			);

		}

		// menu options.
		if ( $screen->id === $this->screen_id_options ) {

			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-framework-options',
				$this->shared->get( 'url' ) . 'admin/assets/css/framework/options.css',
				array(),
				$this->shared->get( 'ver' )
			);
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-jquery-ui-tooltip',
				$this->shared->get( 'url' ) . 'admin/assets/css/jquery-ui-tooltip.css',
				array(),
				$this->shared->get( 'ver' )
			);

			// Select2.
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-select2',
				$this->shared->get( 'url' ) . 'admin/assets/inc/select2/css/select2.min.css',
				array(),
				$this->shared->get( 'ver' )
			);
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-select2-custom',
				$this->shared->get( 'url' ) . 'admin/assets/css/select2-custom.css',
				array(),
				$this->shared->get( 'ver' )
			);

			// WP Color Picker.
			wp_enqueue_style( 'wp-color-picker' );

		}
	}

	/**
	 * Enqueue admin JavaScript.
	 *
	 * @return void
	 */
	public function enqueue_admin_scripts() {

		$wp_localize_script_data = array(
			'deleteText'         => strip_tags( __( 'Delete', 'helpful-pro' ) ),
			'cancelText'         => strip_tags( __( 'Cancel', 'helpful-pro' ) ),
			'chooseAnOptionText' => strip_tags( __( 'Choose an Option ...', 'helpful-pro' ) ),
			'closeText'          => strip_tags( __( 'Close', 'helpful-pro' ) ),
			'postText'           => strip_tags( __( 'Post', 'helpful-pro' ) ),
			'itemsText'          => strip_tags( __( 'items', 'helpful-pro' ) ),
			'dateTooltipText'    => strip_tags( __( 'The date of the feedback.', 'helpful-pro' ) ),
			'ratingTooltipText'  => strip_tags( __( 'The rating received by the feedback.', 'helpful-pro' ) ),
			'commentTooltipText' => strip_tags( __( 'The comment associated with the feedback.', 'helpful-pro' ) ),
		);

		$screen = get_current_screen();

		// menu statistics.
		if ( $screen->id == $this->screen_id_statistics ) {

			// jQuery UI Tooltips.
			wp_enqueue_script( 'jquery-ui-tooltip' );
			wp_enqueue_script(
				$this->shared->get( 'slug' ) . '-jquery-ui-tooltip-init',
				$this->shared->get( 'url' ) . 'admin/assets/js/jquery-ui-tooltip-init.js',
				array( 'jquery' ),
				$this->shared->get( 'ver' )
			);

			// Select2.
			wp_enqueue_script(
				$this->shared->get( 'slug' ) . '-select2',
				$this->shared->get( 'url' ) . 'admin/assets/inc/select2/js/select2.min.js',
				array( 'jquery' ),
				$this->shared->get( 'ver' )
			);

			// Menu Statistics.
			wp_enqueue_script(
				$this->shared->get( 'slug' ) . '-menu-statistics',
				$this->shared->get( 'url' ) . 'admin/assets/js/menu-statistics.js',
				array(
					'jquery',
					'jquery-ui-dialog',
					'daexthefup-select2',
					'jquery-ui-tooltip',
					'daexthefup-jquery-ui-tooltip-init',
				),
				$this->shared->get( 'ver' )
			);

			wp_localize_script(
				$this->shared->get( 'slug' ) . '-menu-statistics',
				'objectL10n',
				$wp_localize_script_data
			);

			// Store the JavaScript parameters in the window.DAEXTHEFUP_PARAMETERS object.
			$script  = 'window.DAEXTHEFUP_PARAMETERS = {';
			$script .= 'ajaxUrl: "' . admin_url( 'admin-ajax.php' ) . '",';
			$script .= 'nonce: "' . wp_create_nonce( 'daexthefup' ) . '",';
			$script .= 'adminUrl: "' . get_admin_url() . '",';
			$script .= '};';
			if ( false !== $script ) {
				wp_add_inline_script( $this->shared->get( 'slug' ) . '-menu-statistics', $script, 'before' );
			}
		}

		// Menu Maintenance.
		if ( $screen->id == $this->screen_id_maintenance ) {

			// Maintenance Menu.
			wp_enqueue_script(
				$this->shared->get( 'slug' ) . '-menu-maintenance',
				$this->shared->get( 'url' ) . 'admin/assets/js/menu-maintenance.js',
				array( 'jquery', 'jquery-ui-dialog' ),
				$this->shared->get( 'ver' )
			);
			wp_localize_script(
				$this->shared->get( 'slug' ) . '-menu-maintenance',
				'objectL10n',
				$wp_localize_script_data
			);

			// jQuery UI Tooltip.
			wp_enqueue_script( 'jquery-ui-tooltip' );
			wp_enqueue_script(
				$this->shared->get( 'slug' ) . '-jquery-ui-tooltip-init',
				$this->shared->get( 'url' ) . 'admin/assets/js/jquery-ui-tooltip-init.js',
				array( 'jquery' ),
				$this->shared->get( 'ver' )
			);

			// Select2.
			wp_enqueue_script(
				$this->shared->get( 'slug' ) . '-select2',
				$this->shared->get( 'url' ) . 'admin/assets/inc/select2/js/select2.min.js',
				array( 'jquery' ),
				$this->shared->get( 'ver' )
			);

		}

		// menu options.
		if ( $screen->id == $this->screen_id_options ) {

			// jQuery UI Tooltips.
			wp_enqueue_script( 'jquery-ui-tooltip' );
			wp_enqueue_script(
				$this->shared->get( 'slug' ) . '-jquery-ui-tooltip-init',
				$this->shared->get( 'url' ) . 'admin/assets/js/jquery-ui-tooltip-init.js',
				array( 'jquery' ),
				$this->shared->get( 'ver' )
			);

			// Select2.
			wp_enqueue_script(
				$this->shared->get( 'slug' ) . '-select2',
				$this->shared->get( 'url' ) . 'admin/assets/inc/select2/js/select2.min.js',
				array( 'jquery' ),
				$this->shared->get( 'ver' )
			);

			// Color Picker Initialization.
			wp_enqueue_script(
				$this->shared->get( 'slug' ) . '-wp-color-picker-init',
				$this->shared->get( 'url' ) . 'admin/assets/js/wp-color-picker-init.js',
				array( 'jquery', 'wp-color-picker' ),
				false,
				true
			);

			// Menu Options.
			wp_enqueue_script(
				$this->shared->get( 'slug' ) . '-menu-options',
				$this->shared->get( 'url' ) . 'admin/assets/js/menu-options.js',
				array(
					'jquery',
				),
				$this->shared->get( 'ver' )
			);

		}

		/**
		 * When the editor file is loaded (only in the post editor) add helpful form statistics as
		 * json data in a property of the window.DAEXTHEFUP_PARAMETERS object.
		 */
		$post_types_with_ui_a = $this->shared->get_post_types_with_ui();
		if ( in_array( $screen->id, $post_types_with_ui_a ) ) {

			$statistics_a = $this->shared->get_feedback_statistics( get_the_ID() );

			// Store the JavaScript parameters in the window.DAEXTHEFUP_PARAMETERS object.
			$initialization_script  = 'window.DAEXTHEFUP_PARAMETERS = {';
			$initialization_script .= 'statistics: ' . json_encode( $statistics_a );
			$initialization_script .= '};';
			wp_add_inline_script( $this->shared->get( 'slug' ) . '-editor-js', $initialization_script, 'before' );

		}
	}

	/*
	 * plugin activation.
	 */
	public function ac_activate( $networkwide ) {

		/**
		 * Create options and tables for all the sites in the network.
		 */
		if ( function_exists( 'is_multisite' ) and is_multisite() ) {

			/**
			 * If this is a "Network Activation" create the options and tables
			 * for each blog.
			 */
			if ( $networkwide ) {

				// get the current blog id.
				global $wpdb;
				$current_blog = $wpdb->blogid;

				// create an array with all the blog ids.
				$blogids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );

				// iterate through all the blogs.
				foreach ( $blogids as $blog_id ) {

					// switch to the iterated blog.
					switch_to_blog( $blog_id );

					// create options and tables for the iterated blog
					$this->ac_initialize_options();
					$this->ac_create_database_tables();
					$this->ac_initialize_custom_css();

				}

				// switch to the current blog.
				switch_to_blog( $current_blog );

			} else {

				/**
				 * if this is not a "Network Activation" create options and
				 * tables only for the current blog.
				 */
				$this->ac_initialize_options();
				$this->ac_create_database_tables();
				$this->ac_initialize_custom_css();

			}
		} else {

			/**
			 * If this is not a multisite installation create options and
			 * tables only for the current blog.
			 */
			$this->ac_initialize_options();
			$this->ac_create_database_tables();
			$this->ac_initialize_custom_css();

		}
	}

	/**
	 * Create the options and tables for the newly created blog.
	 *
	 * @param $blog_id
	 * @param $user_id
	 * @param $domain
	 * @param $path
	 * @param $site_id
	 * @param $meta
	 *
	 * @return void
	 */
	public function new_blog_create_options_and_tables( $blog_id, $user_id, $domain, $path, $site_id, $meta ) {

		global $wpdb;

		/*
		 * if the plugin is "Network Active" create the options and tables for
		 * this new blog
		 */
		if ( is_plugin_active_for_network( 'helpful-pro/init.php' ) ) {

			// get the id of the current blog.
			$current_blog = $wpdb->blogid;

			// switch to the blog that is being activated.
			switch_to_blog( $blog_id );

			// create options and database tables for the new blog.
			$this->ac_initialize_options();
			$this->ac_create_database_tables();
			$this->ac_initialize_custom_css();

			// switch to the current blog.
			switch_to_blog( $current_blog );

		}
	}

	/**
	 * Delete options and tables for the deleted blog.
	 *
	 * @param $blog_id
	 *
	 * @return void
	 */
	public function delete_blog_delete_options_and_tables( $blog_id ) {

		global $wpdb;

		// get the id of the current blog.
		$current_blog = $wpdb->blogid;

		// switch to the blog that is being activated.
		switch_to_blog( $blog_id );

		// create options and database tables for the new blog.
		$this->un_delete_options();
		$this->un_delete_database_tables();

		// switch to the current blog.
		switch_to_blog( $current_blog );
	}

	/**
	 * Initialize plugin options.
	 *
	 * @return void
	 */
	private function ac_initialize_options() {

		foreach ( $this->shared->get( 'options' ) as $key => $value ) {
			add_option( $key, $value );
		}
	}

	/**
	 * Create the plugin database tables.
	 *
	 * @return void
	 */
	private function ac_create_database_tables() {

		global $wpdb;

		// Get the database character collate that will be appended at the end of each query.
		$charset_collate = $wpdb->get_charset_collate();

		// check database version and create the database.
		if ( intval( get_option( $this->shared->get( 'slug' ) . '_database_version' ), 10 ) < 1 ) {

			require_once ABSPATH . 'wp-admin/includes/upgrade.php';

			// create *prefix*_feedback.
			$table_name = $wpdb->prefix . $this->shared->get( 'slug' ) . '_feedback';
			$sql        = "CREATE TABLE $table_name (
                feedback_id bigint(20) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
                date datetime DEFAULT NULL,
                post_id bigint(20) UNSIGNED DEFAULT NULL,
                value tinyint(1) UNSIGNED DEFAULT NULL,
                description text NOT NULL DEFAULT '',
                ip_address text NOT NULL DEFAULT ''
            ) $charset_collate";
			dbDelta( $sql );

			// create *prefix*_archive.
			$table_name = $wpdb->prefix . $this->shared->get( 'slug' ) . '_archive';
			$sql        = "CREATE TABLE $table_name (
                archive_id bigint(20) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
                post_id bigint(20) UNSIGNED,
                post_title text NOT NULL DEFAULT '',
                post_type varchar(20) NOT NULL DEFAULT '',
                post_date datetime DEFAULT NULL,
                positive_feedback bigint(20) UNSIGNED,
                negative_feedback bigint(20) UNSIGNED,
                pfr tinyint(1) DEFAULT -1
            ) $charset_collate";
			dbDelta( $sql );

			// Update database version.
			update_option( $this->shared->get( 'slug' ) . '_database_version', '1' );

		}
	}

	/**
	 * Plugin delete.
	 *
	 * @return void
	 */
	public static function un_delete() {

		/**
		 * Delete options and tables for all the sites in the network.
		 */
		if ( function_exists( 'is_multisite' ) and is_multisite() ) {

			// get the current blog id.
			global $wpdb;
			$current_blog = $wpdb->blogid;

			// create an array with all the blog ids.
			$blogids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );

			// iterate through all the blogs.
			foreach ( $blogids as $blog_id ) {

				// switch to the iterated blog.
				switch_to_blog( $blog_id );

				// create options and tables for the iterated blog.
				self::un_delete_options();
				self::un_delete_database_tables();

			}

			// switch to the current blog.
			switch_to_blog( $current_blog );

		} else {

			/**
			 * If this is not a multisite installation delete options and
			 * tables only for the current blog.
			 */
			self::un_delete_options();
			self::un_delete_database_tables();

		}
	}

	/**
	 * Delete plugin options.
	 *
	 * @return void
	 */
	public static function un_delete_options() {

		// assign an instance of Daexthefup_Shared.
		$shared = Daexthefup_Shared::get_instance();

		foreach ( $shared->get( 'options' ) as $key => $value ) {
			delete_option( $key );
		}
	}

	/**
	 * Delete plugin database tables.
	 *
	 * @return void
	 */
	public static function un_delete_database_tables() {

		// assign an instance of Daexthefup_Shared.
		$shared = Daexthefup_Shared::get_instance();

		global $wpdb;
		$table_name = $wpdb->prefix . $shared->get( 'slug' ) . '_archive';
		$sql        = "DROP TABLE $table_name";
		$wpdb->query( $sql );

		$table_name = $wpdb->prefix . $shared->get( 'slug' ) . '_feedback';
		$sql        = "DROP TABLE $table_name";
		$wpdb->query( $sql );
	}

	/**
	 * Register the admin menu.
	 *
	 * @return void
	 */
	public function me_add_admin_menu() {

		add_menu_page(
			esc_html__( 'HF', 'helpful-pro' ),
			esc_html__( 'Helpful', 'helpful-pro' ),
			get_option( $this->shared->get( 'slug' ) . '_statistics_menu_capability' ),
			$this->shared->get( 'slug' ) . '-statistics',
			array( $this, 'me_display_menu_statistics' ),
			'dashicons-thumbs-up'
		);

		$this->screen_id_statistics = add_submenu_page(
			$this->shared->get( 'slug' ) . '-statistics',
			esc_html__( 'HF - Statistics', 'helpful-pro' ),
			esc_html__( 'Statistics', 'helpful-pro' ),
			get_option( $this->shared->get( 'slug' ) . '_statistics_menu_capability' ),
			$this->shared->get( 'slug' ) . '-statistics',
			array( $this, 'me_display_menu_statistics' )
		);

		$this->screen_id_maintenance = add_submenu_page(
			$this->shared->get( 'slug' ) . '-statistics',
			esc_html__( 'HF - Maintenance', 'helpful-pro' ),
			esc_html__( 'Maintenance', 'helpful-pro' ),
			get_option( $this->shared->get( 'slug' ) . '_maintenance_menu_capability' ),
			$this->shared->get( 'slug' ) . '-maintenance',
			array( $this, 'me_display_menu_maintenance' )
		);

		$this->screen_id_help = add_submenu_page(
			$this->shared->get( 'slug' ) . '-statistics',
			esc_html__( 'HF - Help', 'helpful-pro' ),
			esc_html__( 'Help', 'helpful-pro' ),
			'manage_options',
			$this->shared->get( 'slug' ) . '-help',
			array( $this, 'me_display_menu_help' )
		);

		$this->screen_id_options = add_submenu_page(
			$this->shared->get( 'slug' ) . '-statistics',
			esc_html__( 'HF - Options', 'helpful-pro' ),
			esc_html__( 'Options', 'helpful-pro' ),
			'manage_options',
			$this->shared->get( 'slug' ) . '-options',
			array( $this, 'me_display_menu_options' )
		);
	}

	/**
	 * Includes the statistics menu.
	 *
	 * @return void
	 */
	public function me_display_menu_statistics() {
		include_once 'view/statistics.php';
	}

	/**
	 * Includes the maintenance menu.
	 *
	 * @return void
	 */
	public function me_display_menu_maintenance() {
		include_once 'view/maintenance.php';
	}

	/**
	 * Includes the form menu.
	 *
	 * @return void
	 */
	public function me_display_menu_help() {
		include_once 'view/help.php';
	}

	/**
	 * Includes the options menu.
	 *
	 * @return void
	 */
	public function me_display_menu_options() {
		include_once 'view/options.php';
	}

	/**
	 * register options
	 *
	 * @return void
	 */
	public function op_register_options() {

		$this->menu_options->register_options();
	}

	/**
	 * Generate the custom.css file based on the values of the options and write them down in the custom.css file.
	 *
	 * @return false|int
	 */
	public function write_custom_css() {

		// turn on output buffering.
		ob_start();

		// Button Font Family.
		echo '.daexthefup-button-text, .daexthefup-comment-submit, .daexthefup-comment-cancel{font-family: ' .
			htmlspecialchars(
				get_option( $this->shared->get( 'slug' ) . '_button_font_family' ),
				ENT_COMPAT
			) . ' !important; }';

		// Feedback Button Font Size.
		echo '.daexthefup-button-text{font-size: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_rating_button_font_size' ) ) . 'px !important; }';

		// Feedback Button Font Style.
		echo '.daexthefup-button-text{font-style: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_rating_button_font_style' ) ) . ' !important; }';

		// Feedback Button Font Weight.
		echo '.daexthefup-button-text{font-weight: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_rating_button_font_weight' ) ) . ' !important; }';

		// Feedback Button Line Height.
		echo '.daexthefup-button-text{line-height: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_rating_button_line_height' ) ) . 'px !important; }';

		// Feedback Button Font Color.
		echo '.daexthefup-button-text{color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_rating_button_font_color' ) ) . ' !important; }';

		// Button Font Size.
		echo '.daexthefup-comment-submit, .daexthefup-comment-cancel{font-size: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_button_font_size' ) ) . 'px !important; }';

		// Button Font Style.
		echo '.daexthefup-comment-submit, .daexthefup-comment-cancel{font-style: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_button_font_style' ) ) . ' !important; }';

		// Button Font Weight.
		echo '.daexthefup-comment-submit, .daexthefup-comment-cancel{font-weight: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_button_font_weight' ) ) . ' !important; }';

		// Button Line Height.
		echo '.daexthefup-comment-submit, .daexthefup-comment-cancel{line-height: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_button_line_height' ) ) . 'px !important; }';

		// Primary Button Background Color.
		echo '.daexthefup-comment-submit{background: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_primary_button_background_color' ) ) . ' !important; }';

		// Primary Button Border Color.
		echo '.daexthefup-comment-submit{border-color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_primary_button_border_color' ) ) . ' !important; }';

		// Secondary Button Background Color.
		echo '.daexthefup-comment-cancel{background: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_secondary_button_background_color' ) ) . ' !important; }';

		// Secondary Button Border Color.
		echo '.daexthefup-comment-cancel{border-color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_secondary_button_border_color' ) ) . ' !important; }';

		// Comment Intro Font Family.
		echo '.daexthefup-comment-label,
		.daexthefup-comment-character-counter-number, .daexthefup-comment-character-counter-text,
		.daexthefup-successful-submission-text{font-family: ' .
			htmlspecialchars(
				get_option( $this->shared->get( 'slug' ) . '_base_font_family' ),
				ENT_COMPAT
			) . ' !important; }';

		// Comment Intro Font Size.
		echo '.daexthefup-comment-label,
		.daexthefup-comment-character-counter-number, .daexthefup-comment-character-counter-text,
		.daexthefup-successful-submission-text{font-size: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_base_font_size' ) ) . 'px !important; }';

		// Comment Intro Font Style.
		echo '.daexthefup-comment-label,
		.daexthefup-comment-character-counter-number, .daexthefup-comment-character-counter-text,
		.daexthefup-successful-submission-text{font-style: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_base_font_style' ) ) . ' !important; }';

		// Comment Intro Font Weight.
		echo '.daexthefup-comment-label,
		.daexthefup-comment-character-counter-number, .daexthefup-comment-character-counter-text,
		.daexthefup-successful-submission-text{font-weight: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_base_font_weight' ) ) . ' !important; }';

		// Comment Intro Line Height.
		echo '.daexthefup-comment-label,
		.daexthefup-comment-character-counter-number, .daexthefup-comment-character-counter-text,
		.daexthefup-successful-submission-text{line-height: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_base_line_height' ) ) . 'px !important; }';

		// Comment Intro Font Color.
		echo '.daexthefup-comment-label{color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_label_font_color' ) ) . ' !important; }';

		// Successful Submission Font Color.
		echo '.daexthefup-successful-submission-text{color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_successful_submission_font_color' ) ) . ' !important; }';

		// Character Counter Font Color.
		echo '.daexthefup-comment-character-counter-number, .daexthefup-comment-character-counter-text{color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_character_counter_font_color' ) ) . ' !important; }';

		// Primary Button Font Color.
		echo '.daexthefup-comment-submit{color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_primary_button_font_color' ) ) . ' !important; }';

		// Secondary Button Font Color.
		echo '.daexthefup-comment-cancel{color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_secondary_button_font_color' ) ) . ' !important; }';

		// Comment Textarea Border Color Selected.
		echo '.daexthefup-comment-textarea:focus{border-color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_comment_textarea_border_color_selected' ) ) . ' !important; }';

		// Comment Textarea Font Family.
		echo '.daexthefup-comment-textarea{font-family: ' .
			htmlspecialchars(
				get_option( $this->shared->get( 'slug' ) . '_comment_textarea_font_family' ),
				ENT_COMPAT
			) . ' !important; }';

		// Comment Textarea Font Size.
		echo '.daexthefup-comment-textarea{font-size: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_comment_textarea_font_size' ) ) . 'px !important; }';

		// Comment Textarea Font Style.
		echo '.daexthefup-comment-textarea{font-style: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_comment_textarea_font_style' ) ) . ' !important; }';

		// Comment Textarea Font Weight.
		echo '.daexthefup-comment-textarea{font-weight: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_comment_textarea_font_weight' ) ) . ' !important; }';

		// Comment Textarea Line Height.
		echo '.daexthefup-comment-textarea{line-height: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_comment_textarea_line_height' ) ) . 'px !important; }';

		// Comment Textarea Font Color.
		echo '.daexthefup-comment-textarea{color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_comment_textarea_font_color' ) ) . ' !important; }';

		// Comment Textarea Background Color.
		echo '.daexthefup-comment-textarea{background: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_comment_textarea_background_color' ) ) . ' !important; }';

		// Comment Textarea Border Color.
		echo '.daexthefup-comment-textarea{border-color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_comment_textarea_border_color' ) ) . ' !important; }';

		// Background.
		if ( intval( get_option( $this->shared->get( 'slug' ) . '_background' ), 10 ) === 1 ) {
			echo '.daexthefup-container{background: ' .
				esc_attr( get_option( $this->shared->get( 'slug' ) . '_background_color' ) ) . ' !important; }';
		}

		// Border.
		$border = intval( get_option( $this->shared->get( 'slug' ) . '_border' ), 10 );
		switch ( $border ) {

			case 0:
				echo '.daexthefup-container{border-width: 0 !important; }';
				break;

			case 1:
				echo '.daexthefup-container{border-width: 1px 0 !important; }';
				break;

			case 2:
				echo '.daexthefup-container{border-width: 0 1px !important; }';
				break;

			case 3:
				echo '.daexthefup-container{border-width: 1px !important; }';

				// The border radius on the container is applied only if the container border is set to "Complete".
				echo '.daexthefup-container{border-radius: ' .
					esc_attr( get_option( $this->shared->get( 'slug' ) . '_border_radius' ) ) . 'px !important; }';

				break;

		}

		echo '.daexthefup-container{border-color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_border_color' ) ) . ' !important; }';

		// Container Horizontal Padding/Container Vertical Padding.
		echo '.daexthefup-container{padding: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_container_vertical_padding' ) ) . 'px ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_container_horizontal_padding' ) ) . 'px ' .
			'!important; }';

		// Container Horizontal Margin/Container Vertical Margin.
		echo '.daexthefup-container{margin: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_container_vertical_margin' ) ) . 'px ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_container_horizontal_margin' ) ) . 'px ' .
			'!important; }';

		// Title Font Family.
		echo '.daexthefup-title{font-family: ' .
			htmlspecialchars(
				get_option( $this->shared->get( 'slug' ) . '_title_font_family' ),
				ENT_COMPAT
			) . ' !important; }';

		// Title Font Size.
		echo '.daexthefup-title{font-size: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_title_font_size' ) ) . 'px !important; }';

		// Title Font Style.
		echo '.daexthefup-title{font-style: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_title_font_style' ) ) . ' !important; }';

		// Title Font Weight.
		echo '.daexthefup-title{font-weight: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_title_font_weight' ) ) . ' !important; }';

		// Title Line Height.
		echo '.daexthefup-title{line-height: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_title_line_height' ) ) . 'px !important; }';

		// Title Font Color.
		echo '.daexthefup-title{color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_title_font_color' ) ) . ' !important; }';

		// Description Font Family.
		echo '.daexthefup-description{font-family: ' .
			htmlspecialchars(
				get_option( $this->shared->get( 'slug' ) . '_description_font_family' ),
				ENT_COMPAT
			) . ' !important; }';

		// Description Font Size.
		echo '.daexthefup-description{font-size: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_description_font_size' ) ) . 'px !important; }';

		// Description Font Style.
		echo '.daexthefup-description{font-style: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_description_font_style' ) ) . ' !important; }';

		// Description Font Weight.
		echo '.daexthefup-description{font-weight: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_description_font_weight' ) ) . ' !important; }';

		// Description Line Height.
		echo '.daexthefup-description{line-height: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_description_line_height' ) ) . 'px !important; }';

		// Description Font Color.
		echo '.daexthefup-description{color: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_description_font_color' ) ) . 'px !important; }';

		// Button Icon Primary Color.
		echo '.happy-face-cls-1, .sad-face-cls-1, .thumb-up-cls-1, .thumb-down-cls-1{fill: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_button_icon_primary_color' ) ) . ' !important; }';

		// Button Icon Secondary Color.
		echo '.happy-face-cls-3, .sad-face-cls-4, .thumb-up-cls-3, .thumb-down-cls-3{fill: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_button_icon_secondary_color' ) ) . ' !important; }';

		// Button Icon Primary Color Positive Selected.
		echo '.daexthefup-yes:hover .daexthefup-icon-primary-color,
		.daexthefup-yes-selected .daexthefup-icon-primary-color' .
			'{fill: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_button_icon_primary_color_positive_selected' ) ) . ' !important }';

		// Button Icon Secondary Color Positive Selected.
		echo '.daexthefup-yes:hover .daexthefup-icon-secondary-color,' .
			'.daexthefup-yes-selected .daexthefup-icon-secondary-color' .
			'{fill:' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_button_icon_secondary_color_positive_selected' ) ) . ' !important }';

		// Button Icon Primary Color Negative Selected.
		echo '.daexthefup-no:hover .daexthefup-icon-primary-color,
			.daexthefup-no-selected .daexthefup-icon-primary-color' .
			'{fill: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_button_icon_primary_color_negative_selected' ) ) . ' !important }';

		// Button Icon Secondary Color Negative Selected.
		echo '.daexthefup-no:hover .daexthefup-icon-secondary-color,' .
			'.daexthefup-no-selected .daexthefup-icon-secondary-color' .
			'{fill:' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_button_icon_secondary_color_negative_selected' ) ) . ' !important }';

		// Positive Feedback Button Background Color.
		echo '.daexthefup-button-type-icon-and-text.daexthefup-button.daexthefup-yes,
		.daexthefup-button-type-text.daexthefup-button.daexthefup-yes,
		.daexthefup-button-type-icon-and-text.daexthefup-button.daexthefup-no,
		.daexthefup-button-type-text.daexthefup-button.daexthefup-no{background: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_rating_button_background_color' ) ) . ' !important; }';

		// Button Icons Border Color.
		echo '.daexthefup-icon-circle{fill: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_button_icons_border_color' ) ) . ' !important; }';

		// Border Radius.
		echo '.daexthefup-button,
		.daexthefup-comment-textarea{border-radius: ' .
			esc_attr( get_option( $this->shared->get( 'slug' ) . '_border_radius' ) ) . 'px !important; }';

		$custom_css_string = ob_get_clean();

		// Get the upload directory path and the file path.
		$upload_dir_path  = $this->get_plugin_upload_path();
		$upload_file_path = $this->get_plugin_upload_path() . 'custom-' . get_current_blog_id() . '.css';

		// If the plugin upload directory doesn't exist create it.
		if ( ! is_dir( $upload_dir_path ) ) {
			mkdir( $upload_dir_path );
		}

		// Write the custom css file.
		return @file_put_contents(
			$upload_file_path,
			$custom_css_string,
			LOCK_EX
		);
	}

	/**
	 * Initialize the custom-[blog_id].css file.
	 *
	 * @return void
	 */
	public function ac_initialize_custom_css() {

		/**
		 * Write the custom-[blog_id].css file or die if the file can't be created or modified.
		 */
		if ( $this->write_custom_css() === false ) {
			die( "The plugin can't write files in the upload directory." );
		}
	}

	/**
	 * Get the plugin upload path.
	 *
	 * @return string The plugin upload path
	 */
	public function get_plugin_upload_path() {

		$upload_path = WP_CONTENT_DIR . '/uploads/daexthefup_uploads/';

		return $upload_path;
	}

	/**
	 * Echo all the dismissible notices based on the values of the $notices array.
	 *
	 * @param $notices
	 */
	public function dismissible_notice( $notices ) {

		foreach ( $notices as $notice ) {
			echo '<div class="' . esc_attr( $notice['class'] ) . ' settings-error notice is-dismissible below-h2"><p>' . esc_html( $notice['message'] ) . '</p></div>';
		}
	}

	/**
	 * Add custom columns to all the post types with a public UI.
	 *
	 * See: https://wordpress.stackexchange.com/questions/253640/adding-custom-columns-to-custom-post-types
	 */
	public function add_custom_columns() {

		$post_types_with_ui_a = $this->shared->get_post_types_with_ui();
		foreach ( $post_types_with_ui_a as $post_type ) {
			add_filter( 'manage_' . $post_type . '_posts_columns', array( $this, 'set_custom_edit_post_columns' ) );
			add_action( 'manage_' . $post_type . '_posts_custom_column', array( $this, 'custom_columns' ), 10, 2 );
		}
	}

	/**
	 * Add the custom columns to the "post" post type.
	 *
	 * @param $columns
	 *
	 * @return mixed
	 */
	function set_custom_edit_post_columns( $columns ) {

		$columns['daexthefup_statistics'] = __( 'Feedback', 'helpful-pro' );

		return $columns;
	}

	/**
	 * Add the data to the custom columns for the "post" post type.
	 *
	 * @param $column
	 * @param $post_id
	 */
	function custom_columns( $column, $post_id ) {

		switch ( $column ) {

			case 'daexthefup_statistics':
				$statistics = $this->shared->get_feedback_statistics( $post_id );

				if ( $statistics['positive_value'] > 0 || $statistics['negative_value'] > 0 ) {
					$positive_percentage_value_text = $statistics['positive_percentage_value'] . '%';
				} else {
					$positive_percentage_value_text = 'N/A';
				}

				echo '<div class="daexthefup-positive-percentage-value">' . esc_html( $positive_percentage_value_text ) . '</div>';
				echo '<div class="daexthefup-positive-vs-negative-values">' .
					esc_html( $statistics['positive_value'] ) . ' ' . esc_html__( 'helpful', 'helpful-pro' ) .
					' / ' . esc_html( $statistics['negative_value'] ) . ' ' . esc_html__( 'not helpful', 'helpful-pro' ) .
					'</div>';
				echo '<div class="daexthefup-positive-vs-negative-bar-container">';
				echo '<div class="daexthefup-positive-vs-negative-bar-positive" style="width: ' . esc_attr( $statistics['positive_percentage_value'] ) . '%"></div>';
				echo '</div>';

				break;

		}
	}
}
