<?php
/**
 * Options page.
 *
 * @package helpful-pro
 */

if ( ! current_user_can( 'manage_options' ) ) {
	wp_die( esc_attr__( 'You do not have sufficient capabilities to access this page.', 'helpful-pro' ) );
}

// Sanitization -------------------------------------------------------------------------------------------------.
$data['settings_updated'] = isset( $_GET['settings-updated'] ) ? sanitize_key( $_GET['settings-updated'], 10 ) : null;
$data['active_tab']       = isset( $_GET['tab'] ) ? sanitize_key( $_GET['tab'] ) : 'content_options';

?>

<div class="wrap">

	<h2><?php esc_html_e( 'Helpful - Options', 'helpful-pro' ); ?></h2>

	<?php

	// settings errors.
	if ( ! is_null( $data['settings_updated'] ) and $data['settings_updated'] == 'true' ) {

		if ( $this->write_custom_css() === false ) {
			?>
			<div id="setting-error-settings_updated" class="error settings-error notice is-dismissible below-h2">
				<p><strong><?php esc_html_e( "The plugin can't write files in the upload directory.", 'helpful-pro' ); ?></strong></p>
				<button type="button" class="notice-dismiss"><span
							class="screen-reader-text"><?php esc_html_e( 'Dismiss this notice.', 'helpful-pro' ); ?></span></button>
			</div>
			<?php
		}

		// Settings errors.
		settings_errors();

	}

	?>

	<div id="daext-options-wrapper">

		<div class="nav-tab-wrapper">
			<a href="?page=daexthefup-options&tab=content_options"
				class="nav-tab <?php echo 'content_options' === $data['active_tab'] ? 'nav-tab-active' : ''; ?>"><?php esc_html_e( 'Content', 'helpful-pro' ); ?></a>
			<a href="?page=daexthefup-options&tab=fonts_options"
				class="nav-tab <?php echo 'fonts_options' === $data['active_tab'] ? 'nav-tab-active' : ''; ?>"><?php esc_html_e( 'Fonts', 'helpful-pro' ); ?></a>
			<a href="?page=daexthefup-options&tab=colors_options"
				class="nav-tab <?php echo 'colors_options' === $data['active_tab'] ? 'nav-tab-active' : ''; ?>"><?php esc_html_e( 'Colors', 'helpful-pro' ); ?></a>
			<a href="?page=daexthefup-options&tab=spacing_options"
				class="nav-tab <?php echo 'spacing_options' === $data['active_tab'] ? 'nav-tab-active' : ''; ?>"><?php esc_html_e( 'Spacing', 'helpful-pro' ); ?></a>
			<a href="?page=daexthefup-options&tab=analysis_options"
				class="nav-tab <?php echo 'analysis_options' === $data['active_tab'] ? 'nav-tab-active' : ''; ?>"><?php esc_html_e( 'Analysis', 'helpful-pro' ); ?></a>
			<a href="?page=daexthefup-options&tab=capabilities_options"
				class="nav-tab <?php echo 'capabilities_options' === $data['active_tab'] ? 'nav-tab-active' : ''; ?>"><?php esc_html_e( 'Capabilities', 'helpful-pro' ); ?></a>
			<a href="?page=daexthefup-options&tab=advanced_options"
				class="nav-tab <?php echo 'advanced_options' === $data['active_tab'] ? 'nav-tab-active' : ''; ?>"><?php esc_html_e( 'Advanced', 'helpful-pro' ); ?></a>
		</div>

		<form method='post' action='options.php' autocomplete="off">

			<?php

			if ( 'content_options' === $data['active_tab'] ) {

				settings_fields( $this->shared->get( 'slug' ) . '_content_options' );
				do_settings_sections( $this->shared->get( 'slug' ) . '_content_options' );

			}

			if ( 'fonts_options' === $data['active_tab'] ) {

				settings_fields( $this->shared->get( 'slug' ) . '_fonts_options' );
				do_settings_sections( $this->shared->get( 'slug' ) . '_fonts_options' );

			}

			if ( 'colors_options' === $data['active_tab'] ) {

				settings_fields( $this->shared->get( 'slug' ) . '_colors_options' );
				do_settings_sections( $this->shared->get( 'slug' ) . '_colors_options' );

			}

			if ( 'spacing_options' === $data['active_tab'] ) {

				settings_fields( $this->shared->get( 'slug' ) . '_spacing_options' );
				do_settings_sections( $this->shared->get( 'slug' ) . '_spacing_options' );

			}

			if ( 'analysis_options' === $data['active_tab'] ) {

				settings_fields( $this->shared->get( 'slug' ) . '_analysis_options' );
				do_settings_sections( $this->shared->get( 'slug' ) . '_analysis_options' );

			}

			if ( 'capabilities_options' === $data['active_tab'] ) {

				settings_fields( $this->shared->get( 'slug' ) . '_capabilities_options' );
				do_settings_sections( $this->shared->get( 'slug' ) . '_capabilities_options' );

			}

			if ( 'advanced_options' === $data['active_tab'] ) {

				settings_fields( $this->shared->get( 'slug' ) . '_advanced_options' );
				do_settings_sections( $this->shared->get( 'slug' ) . '_advanced_options' );

			}

			?>

			<div class="daext-options-action">
				<input type="submit" name="submit" id="submit" class="button"
						value="<?php esc_attr_e( 'Save Changes', 'helpful-pro' ); ?>">
			</div>

		</form>

	</div>

</div>