<?php
/**
 * This class should be used to work with the public side of WordPress.
 *
 * @package helpful-pro
 */

/**
 * This class should be used to work with the public side of WordPress.
 */
class daexthefup_Public {

	protected static $instance = null;
	private $shared            = null;

	private static $shortcode_counter = 0;

	private function __construct() {

		// assign an instance of the plugin info.
		$this->shared = daexthefup_Shared::get_instance();

		// Add the feedback form at the end of the post content.
		add_filter( 'the_content', array( $this, 'add_content' ) );

		// helpful shortcode.
		add_shortcode( 'helpful', array( $this, 'get_helpful_shortcode' ) );

		// Load public css.
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ) );

		// Load public js.
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
	}

	/**
	 * create an instance of this class.
	 *
	 * @return self|null
	 */
	public static function get_instance() {

		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Enqueue the public stylesheets.
	 *
	 * @return void
	 */
	public function enqueue_styles() {

		// Adds the Google Fonts if they are defined in the "Google Font URL" option.
		if ( strlen( trim( get_option( $this->shared->get( 'slug' ) . '_google_font_url' ) ) ) > 0 ) {
			wp_enqueue_style(
				$this->shared->get( 'slug' ) . '-google-font',
				esc_url( get_option( $this->shared->get( 'slug' ) . '_google_font_url' ) ),
				false
			);
		}

		// Enqueue the main stylesheet.
		wp_enqueue_style(
			$this->shared->get( 'slug' ) . '-general',
			$this->shared->get( 'url' ) . 'public/assets/css/general.css',
			array(),
			$this->shared->get( 'ver' )
		);

		// Enqueue the custom CSS file based on the plugin options.
		$upload_dir_data = wp_upload_dir();
		wp_enqueue_style(
			$this->shared->get( 'slug' ) . '-custom',
			$upload_dir_data['baseurl'] . '/daexthefup_uploads/custom-' . get_current_blog_id() . '.css',
			array(),
			$this->shared->get( 'ver' )
		);
	}

	/**
	 * Enqueue the public JavaScript files.
	 *
	 * @return void
	 */
	public function enqueue_scripts() {

		if ( 0 === intval( get_option( $this->shared->get( 'slug' ) . '_assets_mode' ), 10 ) ) {

			// Development.
			wp_enqueue_script(
				$this->shared->get( 'slug' ) . '-utility',
				$this->shared->get( 'url' ) . 'public/assets/js/dev/utility.js',
				array(),
				$this->shared->get( 'ver' ),
				true
			);
			wp_enqueue_script(
				$this->shared->get( 'slug' ) . '-general',
				$this->shared->get( 'url' ) . 'public/assets/js/dev/general.js',
				array( 'jquery', $this->shared->get( 'slug' ) . '-utility' ),
				$this->shared->get( 'ver' ),
				true
			);

			$partial_script_handle = 'general';

		} else {

			// Production.
			wp_enqueue_script(
				$this->shared->get( 'slug' ) . '-main',
				$this->shared->get( 'url' ) . 'public/assets/js/main.js',
				array( 'jquery' ),
				$this->shared->get( 'ver' ),
				true
			);

			$partial_script_handle = 'main';

		}

		// Store the JavaScript parameters in the window.DAEXTHEFUP_PARAMETERS object.
		wp_add_inline_script(
			$this->shared->get( 'slug' ) . '-' . $partial_script_handle,
			'window.DAEXTHEFUP_PHPDATA = ' . json_encode(
				array(
					'ajaxUrl'                       => admin_url( 'admin-ajax.php' ),
					'nonce'                         => wp_create_nonce( 'daexthefup' ),
					'uniqueSubmission'              => intval(
						get_option( $this->shared->get( 'slug' ) . '_unique_submission' ),
						10
					),
					'cookieExpiration'              => $this->shared->get_cookie_expiration_seconds( get_option( $this->shared->get( 'slug' ) . '_cookie_expiration' ) ),
					'commentForm'                   => intval(
						get_option( $this->shared->get( 'slug' ) . '_comment_form' ),
						10
					),
					'textareaCharacters'            => intval(
						get_option( $this->shared->get( 'slug' ) . '_textarea_characters' ),
						10
					),
					'textareaLabelPositiveFeedback' => get_option( $this->shared->get( 'slug' ) . '_comment_form_textarea_label_positive_feedback' ),
					'textareaLabelNegativeFeedback' => get_option( $this->shared->get( 'slug' ) . '_comment_form_textarea_label_negative_feedback' ),
				)
			),
			'before'
		);
	}

	public function add_content( $content ) {

		$form = '';

		// Do not proceed if the helpful form is present in the content.
		if ( strpos( $content, '[helpful' ) !== false ) {
			return $content;
		}

		// Get the list of post types where the form should be applied.
		$post_types_a = maybe_unserialize( get_option( $this->shared->get( 'slug' ) . '_post_types' ) );

		// Verify the post type.
		if ( is_array( $post_types_a ) and in_array( get_post_type(), $post_types_a ) ) {

			$form = $this->get_form_html();

		}

		// Place the form in the specified position.
		if ( intval( get_option( $this->shared->get( 'slug' ) . '_position' ), 10 ) === 0 ) {
			return $form . $content;
		} else {
			return $content . $form;
		}
	}

	/**
	 * [helpful] shortcode callback.
	 *
	 * @return false|string
	 */
	public function get_helpful_shortcode() {

		++self::$shortcode_counter;

		if ( self::$shortcode_counter > 1 ) {
			return '<p>' . esc_html__( "You can't add multiple feedback forms in the same post.", 'daext-helpful' ) . '</p>';
		} else {
			return $this->get_form_html();
		}
	}

	/**
	 * Generate the HTML of the form.
	 *
	 * @return false|string
	 */
	public function get_form_html() {

		$form = '';

		$post_id = get_the_ID();

		// Do not display the form if the test mode is enabled and the current user is not the administrator.
		if ( intval( get_option( $this->shared->get( 'slug' ) . '_test_mode' ), 10 ) === 1 and
			! current_user_can( 'manage_options' ) ) {
			return '';
		}

		// Do not display the form if it's disabled in this post.
		if ( intval( get_post_meta( $post_id, '_helpful_pro_status', true ), 10 ) === 0 ) {
			return '';
		}

		/**
		 * Do not display the form if the "Required Capability" option is not empty and the user doesn't have the
		 * specified capability.
		 */
		$required_capability = get_option( $this->shared->get( 'slug' ) . '_required_capability' );
		if ( strlen( trim( $required_capability ) ) > 0 && ! current_user_can( $required_capability ) ) {
			return '';
		}

		/**
		 * Do not display the form if the post is older than the specified number of days.
		 */
		$automatically_hide                = intval( get_option( $this->shared->get( 'slug' ) . '_automatically_hide' ), 10 );
		$automatically_hide_number_of_days = intval( get_option( $this->shared->get( 'slug' ) . '_automatically_hide_number_of_days' ), 10 );
		if ( 1 === $automatically_hide &&
			$automatically_hide_number_of_days > 0 &&
			( time() - get_the_time( 'U', $post_id ) ) > ( $automatically_hide_number_of_days * 24 * 60 * 60 ) ) {
			return '';
		}

		// Verify if the user has already submitted this form in this post.
		if ( $this->shared->is_unique_submission( $post_id ) ) {

			// Create a class name for the layout type.
			$layout_class = intval(
				get_option( $this->shared->get( 'slug' ) . '_layout' ),
				10
			) === 0 ? 'daexthefup-layout-side-by-side' : 'daexthefup-layout-stacked';

			// Create a class name for the alignment.
			switch ( intval( get_option( $this->shared->get( 'slug' ) . '_alignment' ), 10 ) ) {

				case 0:
					$alignment_class = 'daexthefup-alignment-left';
					break;

				case 1:
					$alignment_class = 'daexthefup-alignment-center';
					break;

				case 2:
					$alignment_class = 'daexthefup-alignment-right';
					break;

			}

			// turn on output buffering.
			ob_start();

			?>

			<div id="daexthefup-container"
				class="daexthefup-container <?php echo esc_attr( $layout_class ); ?> <?php echo esc_attr( $alignment_class ); ?>"
				data-post-id="<?php echo esc_attr( get_the_ID() ); ?>">

				<div class="daexthefup-feedback">
					<div class="daexthefup-text">
						<h3 class="daexthefup-title"><?php echo esc_html( get_option( $this->shared->get( 'slug' ) . '_title' ) ); ?></h3>
					</div>
					<div class="daexthefup-buttons-container">
						<div class="daexthefup-buttons">
							<?php $this->shared->generated_button_html( true ); ?>
							<?php $this->shared->generated_button_html( false ); ?>
						</div>
					</div>
				</div>

				<div class="daexthefup-comment">
					<div class="daexthefup-comment-top-container">
						<label id="daexthefup-comment-label" class="daexthefup-comment-label"></label>
						<?php
						if ( intval(
							get_option( $this->shared->get( 'slug' ) . '_character_counter' ),
							10
						) === 1 ) :
							?>
							<div class="daexthefup-comment-character-counter-container">
								<div id="daexthefup-comment-character-counter-number"
									class="daexthefup-comment-character-counter-number"></div>
								<div class="daexthefup-comment-character-counter-text"></div>
							</div>
						<?php endif; ?>
					</div>
					<textarea id="daexthefup-comment-textarea" class="daexthefup-comment-textarea"
								placeholder="<?php echo esc_attr( get_option( $this->shared->get( 'slug' ) . '_comment_form_textarea_placeholder' ) ); ?>"
								maxlength="
								<?php
								echo intval(
									get_option( $this->shared->get( 'slug' ) . '_textarea_characters' ),
									10
								);
								?>
									"></textarea>
					<div class="daexthefup-comment-buttons-container">
						<button class="daexthefup-comment-submit daexthefup-button"><?php echo esc_html( get_option( $this->shared->get( 'slug' ) . '_comment_form_button_submit_text' ) ); ?></button>
						<button class="daexthefup-comment-cancel daexthefup-button"><?php echo esc_html( get_option( $this->shared->get( 'slug' ) . '_comment_form_button_cancel_text' ) ); ?></button>
					</div>
				</div>

				<div class="daexthefup-successful-submission-text"><?php echo esc_html( get_option( $this->shared->get( 'slug' ) . '_successful_submission_text' ) ); ?></div>

			</div>

			<?php

			$form = ob_get_clean();

		}

		return $form;
	}
}